/***************************************************************************
                          HTMLPage.h  -  description
                             -------------------
    begin                : Fri Jan 5 2001
    copyright            : (C) 2001 by Sauro Menchetti
    email                : menco@inwind.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// HTMLPage.h

#ifndef HTMLPAGE_H
#define HTMLPAGE_H

#include <map>
#include <string>
#include <vector>
#include "Vocabulary.h"

class HTMLPage
{
private:
	// file name of html page
	std::string pageFile;

	///////////////
	// HTML PAGE //
	///////////////

    // the html page is a string
    std::string htmlPage;
	
	// vector of token of the html page
	std::vector<std::string> tokenVector;

	///////////////////////////
	// SECTIONS OF HTML PAGE //
	///////////////////////////

	// title of html page: <title>...</title>
	std::string htmlPage_title;

	// meta of html page: <meta name.../>
	std::string htmlPage_meta;

	// words between h1: <h1>...</h1>
	std::string htmlPage_h1;

	// words between h2: <h2>...</h2>
	std::string htmlPage_h2;

	// words between h3: <h3>...</h3>
	std::string htmlPage_h3;

	// links in the html page: <a href...>...</a> (link and email)
	std::string htmlPage_link;

	// plain text in the html page
	std::string htmlPage_plaintext;

	//////////////////////////////////
	// VECTORS OF TOKEN OF SECTIONS //
	//////////////////////////////////
        	
	// vector of token of the html page title
	std::vector<std::string> tokenVector_title;

	// vector of token of the html page meta
	std::vector<std::string> tokenVector_meta;
	
	// vector of token of the html page h1
	std::vector<std::string> tokenVector_h1;

	// vector of token of the html page h2
	std::vector<std::string> tokenVector_h2;

	// vector of token of the html page h3
	std::vector<std::string> tokenVector_h3;

	// vector of token of the html page links
	std::vector<std::string> tokenVector_link;

	// vector of token of the html page plain text
	std::vector<std::string> tokenVector_plaintext;

	/////////////////////////
	// WEIGHTS OF HTML TAGS //
	/////////////////////////

	// weights of html tags
	std::map<std::string, double> weight;

	//////////////////
	// BAG OF WORDS //
	//////////////////

	// rappresentation of html page as a set of pair (word, weight), where the word is the vector index found by the vocabulary
	std::vector<double> bag_of_words;

// private methods

    // read html page from file
    void readHtmlPage();

    // private default constructor
	HTMLPage();

public:
	// constructor
    HTMLPage(std::string pageFile) :

		pageFile(pageFile),

		htmlPage(),
		tokenVector(),

		htmlPage_title(),
		htmlPage_meta(),
		htmlPage_h1(),
		htmlPage_h2(),
		htmlPage_h3(),
		htmlPage_link(),
		htmlPage_plaintext(),

        tokenVector_title(),
        tokenVector_meta(),
		tokenVector_h1(),
		tokenVector_h2(),
		tokenVector_h3(),
		tokenVector_link(),
		tokenVector_plaintext(),

		weight(),

		bag_of_words()

	{
		weight["title"]     = 5.0;
		weight["meta"]      = 4.0;
		weight["h1"]        = 4.0;
		weight["h2"]        = 3.0;
		weight["h3"]        = 2.0;
		weight["link"]      = 4.0;
		weight["plaintext"] = 1.0;

		readHtmlPage();
	}

    // destroyer
	~HTMLPage() {}

	// erase the html tag of the page
	void eraseHtmlTag(std::string& section);

	// erase not alphabetic characters
	void eraseNotAlpha(std::string& section);

	// case insensitive
	void caseInsensitive(std::string& section);

	// split the html page in token
	void makeToken(std::string section, std::vector<std::string>& setToken);

	// erase the word from a stop list
	void eraseStopWords(std::string stopWordsFileName);

	// use the stemming algorithm on the word of html pagr
	void stemming(std::vector<std::string>& setToken);

	// call the functions for processing the html page for create the vocabulary
	void processing_for_create_vocabulary(std::string stopWordsFileName);

	// allocate bag of words
	void allocateBagOfWords(unsigned int vocabulary_word_number);

	// find the words between the two tags and return them as a string
	std::string findSection(std::string begin, std::string end, std::string& section);

	// erase the html tag of the tag meta
	void eraseMetaTag(std::string& section);

	// create the bag of words of html page
	void createBagOfWords(const Vocabulary& vocabulary);

	// processing the html page for create the bag of words of the page
	void processing_for_create_bag_of_words(const Vocabulary& vocabulary);

	// get methods
    std::string         	        getPageFile() 	  const { return pageFile;     }

    const std::string&	            getHtmlPage() 	  const { return htmlPage;     }
    const std::vector<std::string>& getTokenVector()  const { return tokenVector;  }

	const std::string& getHtmlPage_title()     const { return htmlPage_title;     }
	const std::string& getHtmlPage_meta()      const { return htmlPage_meta;      }
	const std::string& getHtmlPage_h1()        const { return htmlPage_h1;        }
	const std::string& getHtmlPage_h2()        const { return htmlPage_h2;        }
	const std::string& getHtmlPage_h3()        const { return htmlPage_h3;        }
	const std::string& getHtmlPage_link()      const { return htmlPage_link;      }
	const std::string& getHtmlPage_plaintext() const { return htmlPage_plaintext; }

	const std::vector<std::string>& getTokenVector_title()     const { return tokenVector_title;     }
	const std::vector<std::string>& getTokenVector_meta()      const { return tokenVector_meta;      }
	const std::vector<std::string>& getTokenVector_h1()        const { return tokenVector_h1;        }
	const std::vector<std::string>& getTokenVector_h2()        const { return tokenVector_h2;        }
	const std::vector<std::string>& getTokenVector_h3()        const { return tokenVector_h3;        }
	const std::vector<std::string>& getTokenVector_link()      const { return tokenVector_link;      }
	const std::vector<std::string>& getTokenVector_plaintext() const { return tokenVector_plaintext; }

	const std::map<std::string, double>& getWeight()       const { return weight;       }

	const std::vector<double>&           getBag_of_words() const { return bag_of_words; }
};
	// output
	std::ostream& operator<<(std::ostream& os, const HTMLPage& htmlPage);

#endif // HTMLPAGE_H
