/***************************************************************************
                          Vocabulary.h  -  description
                             -------------------
    begin                : Fri Jan 5 2001
    copyright            : (C) 2001 by Sauro Menchetti
    email                : menco@inwind.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// Vocabulary.h

#ifndef VOCABULARY_H
#define VOCABULARY_H

#include <map>
#include <vector>
#include <string>
#include <cassert>
#include <fstream>

class Vocabulary
{
private:
    // vocabulary vector: for conversions from index to word
    std::vector<std::string> vocabulary_vec;

    // vocabulary map: for conversions from word to index
    std::map<std::string, unsigned int> vocabulary_map;

    // vocabulary file name
    std::string vocabularyFileName;

    // index file name
    std::string indexFileName;

    // stopwords file name
    std::string stopWordsFileName;

// private methods

    // create vocabulary from index of file
    void createVocabulary();

    // write vocabulary to file
    void writeVocabulary();

    // read vocabulary from file
    void readVocabulary();

    // select the right function to call using parameter mode
    void init(std::string mode);

    // private default constructor
	Vocabulary() {}

public:
	// constructor
    Vocabulary(std::string vocabularyFileName, std::string indexFileName, std::string mode, std::string stopWordsFileName = "") :
        vocabulary_vec(),
        vocabulary_map(),
        vocabularyFileName(vocabularyFileName),
        indexFileName(indexFileName),
		stopWordsFileName(stopWordsFileName)
        { init(mode); }

    // destroyer
	~Vocabulary() {}

	// get methods
    const std::vector<std::string>&            getVocabulary_vec()     const { return vocabulary_vec;     }
    const std::map<std::string, unsigned int>& getVocabulary_map()     const { return vocabulary_map;     }
    std::string                                getVocabularyFileName() const { return vocabularyFileName; }
    std::string                                getIndexFileName()      const { return indexFileName;      }
    std::string                                getStopWordsFileName()  const { return stopWordsFileName;  }
};
	// output
	std::ostream& operator<<(std::ostream& os, const Vocabulary& vocabulary);

#endif // VOCABULARY_H
