/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : gio dic 28 17:56:51 CET 2000
    copyright            : (C) 2000 by Sauro Menchetti
    email                : menco@inwind.it
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <exception>
#include <iostream>
#include <cstdlib>
#include <cmath>
#include <string>

#include "HTMLPage.h"
#include "Vocabulary.h"

// parse the command line
void command_line_parser(int argc, char *argv[]);

// compute cos of angle between two vectors
double compute_cos_theta(const std::vector<double>& bag_of_words_1, const std::vector<double>& bag_of_words_2);

int main(int argc, char *argv[])
{
	try
	{
		// parse the command line
		command_line_parser(argc, argv);
    }
    catch (const std::exception& e)
    {
		// processing standard library exception
        std::cerr << "Standard library exception: ";
        std::cerr << e.what() << std::endl;
        return -1;
    }
    catch (...)
    {
        // processing not standard library exception
        std::cerr << "There is a not standard library exception!" << std::endl;
        std::cerr << "return 10\n";
        return -1;
    }

	return EXIT_SUCCESS;
}

void command_line_parser(int argc, char *argv[])
{
	// first argument of the command line
	std::string argv1;
	
	if (argc > 1)
	{
		argv1 = argv[1];
		//std::cout << "argv[1]:#" << argv1 << "#" << std::endl;	
	}

	// CONSTANT STRING FOR THE COMMAND LINE //

	const std::string help              = "--help";

	const std::string create_vocabulary = "--create-vocabulary";
	const std::string voc = "-v";
	const std::string ind = "-i";
	const std::string stw = "-s";

	const std::string compare_html_page = "--compare-html-page";
	const std::string htmlpage1 = "-h1";
	const std::string htmlpage2 = "-h2";

	// HELP FOR COMMAND LINE //

	if ((argc == 1) || (argv1 == help))
	{
		std::cout << std::endl;
		std::cout << "             WCHANGE 2001" << std::endl;
		std::cout << std::endl;
		std::cout << "copyright : (C) 2001 by Sauro Menchetti" << std::endl;		
		std::cout << "email     : menco@inwind.it" << std::endl;		
		std::cout << std::endl;
		std::cout << "This program compares two html pages and says if the content of two pages is changed." << std::endl;
		std::cout << "There are two step:" << std::endl;
		std::cout << "1) first we compute a vocabulary from an index file;" << std::endl;
		std::cout << "2) then we compare the two page using the vocabulary." << std::endl;
		std::cout << std::endl;
		std::cout << "Help for the command line:" << std::endl;
		std::cout << std::endl;
		std::cout << "./wchange --help" << std::endl;
		std::cout << std::endl;
		std::cout << "	Print this help of command line." << std::endl;
		std::cout << std::endl;
		std::cout << "./wchange --create-vocabulary [-v vocabulary-file-name=vocabulary] [-i index-file-name=indexFile] [-s stop-words-file-name=stopwords]" << std::endl;
		std::cout << std::endl;
		std::cout << "	Create the vocabulary for compare two html pages." << std::endl;
		std::cout << std::endl;
		std::cout << "./wchange --compare-html-page [-v vocabulary-file-name=vocabulary] [-h1 html-page-1-file-name=htmlpage1 -h2 html-page-2-file-name=htmlpage1]" << std::endl;
		std::cout << std::endl;
		std::cout << "	Compare two html pages and says if the content is changed." << std::endl;
		std::cout << std::endl;
	}
	else if (argv1 == create_vocabulary)
	{
		// vocabulary file name
		std::string fileVocabularyName = "./vocabulary";

		// file that contains the list of files that are used to create the vocabulary
		std::string indexFile = "./indexFile";

		// stop words file name
		std::string stopWordsFileName = "./stopwords";

		switch (argc)
		{
			case 2:
				// create vocabulary with default names
			break;
			// set one of vocabulary, index file, stopwords
			case 4:
			{
				std::string parameter = argv[2];
				std::string filename  = argv[3];

				if (parameter == voc)
				{
					fileVocabularyName = filename;
				}
				else if (parameter == ind)
				{
					indexFile = filename;
				}
				else if (parameter == stw)
				{
					stopWordsFileName = filename;
				}
				else
				{
					std::cerr << std::endl << "Syntax error: type --help for help" << std::endl << std::endl; exit(-1);
				}
			}	

			break;

			// set two of vocabulary, index file, stopwords
			case 6:
			{
				std::string parameter1 = argv[2];
				std::string filename1  = argv[3];
				std::string parameter2 = argv[4];
				std::string filename2  = argv[5];

				if (parameter1 == voc)
				{
					fileVocabularyName = filename1;
				}
				else if (parameter1 == ind)
				{
					indexFile = filename1;
				}
				else if (parameter1 == stw)
				{
					stopWordsFileName = filename1;
				}
				else
				{
					std::cerr << std::endl << "Syntax error: type --help for help" << std::endl << std::endl; exit(-1);
				}

				if (parameter2 == voc)
				{
					fileVocabularyName = filename2;
				}
				else if (parameter2 == ind)
				{
					indexFile = filename2;
				}
				else if (parameter2 == stw)
				{
					stopWordsFileName = filename2;
				}
				else
				{
					std::cerr << std::endl << "Syntax error: type --help for help" << std::endl << std::endl; exit(-1);
				}
            }

			break;

			// set vocabulary, index file, stopwords
			case 8:
            {
				std::string parameter1 = argv[2];
				std::string filename1  = argv[3];
				std::string parameter2 = argv[4];
				std::string filename2  = argv[5];
				std::string parameter3 = argv[6];
				std::string filename3  = argv[7];

				if (parameter1 == voc)
				{
					fileVocabularyName = filename1;
				}
				else if (parameter1 == ind)
				{
					indexFile = filename1;
				}
				else if (parameter1 == stw)
				{
					stopWordsFileName = filename1;
				}
				else
				{
					std::cerr << std::endl << "Syntax error: type --help for help" << std::endl << std::endl; exit(-1);
				}

				if (parameter2 == voc)
				{
					fileVocabularyName = filename2;
				}
				else if (parameter2 == ind)
				{
					indexFile = filename2;
				}
				else if (parameter2 == stw)
				{
					stopWordsFileName = filename2;
				}
				else
				{
					std::cerr << std::endl << "Syntax error: type --help for help" << std::endl << std::endl; exit(-1);
				}

				if (parameter3 == voc)
				{
					fileVocabularyName = filename3;
				}
				else if (parameter3 == ind)
				{
					indexFile = filename3;
				}
				else if (parameter3 == stw)
				{
					stopWordsFileName = filename3;
				}
				else
				{
					std::cerr << std::endl << "Syntax error: type --help for help" << std::endl << std::endl; exit(-1);
				}
            }

			break;

			default:
			{
				std::cerr << std::endl << "Syntax error: type --help for help" << std::endl << std::endl; exit(-1);
			}

			break;
		}

		std::cout << std::endl;
		std::cout << "Vocabulary file name: " << fileVocabularyName << std::endl;
		std::cout << "Index      file name: " << indexFile          << std::endl;
		std::cout << "Stop Words file name: " << stopWordsFileName  << std::endl;
		std::cout << std::endl;

		// create the vocabulary of word
		Vocabulary vocabulary(fileVocabularyName, indexFile, "w", stopWordsFileName);

		// print the vocabulary
		//std::cout << vocabulary << std::endl;
	}
	else if (argv1 == compare_html_page)
	{
		// vocabulary file name
		std::string fileVocabularyName = "./vocabulary";

		// html page name of the first page
		std::string htmlpagefilename1 = "./htmlpage1";

		// html page name of the second page
		std::string htmlpagefilename2 = "./htmlpage2";

		switch (argc)
		{
			case 2:
				// compare two html pages with default names
			break;
			// set vocabulary
			case 4:
			{
				std::string voc_parameter  = argv[2];
				std::string vocabularyFile = argv[3];

				if (voc_parameter == voc)
				{
					fileVocabularyName = vocabularyFile;		
				}
				else
				{
					std::cerr << std::endl << "Syntax error: type --help for help" << std::endl << std::endl; exit(-1);
				}
			}

			break;

			// set two html pages
			case 6:
			{	
				std::string html1     = argv[2];
				std::string html1file = argv[3];
				std::string html2     = argv[4];
				std::string html2file = argv[5];

				if (html1 == htmlpage1)
				{
					htmlpagefilename1 = html1file;
				}
				else
				{
					std::cerr << std::endl << "Syntax error: type --help for help" << std::endl << std::endl; exit(-1);
				}

				if (html2 == htmlpage2)
				{
					htmlpagefilename2 = html2file;
				}
				else
				{
					std::cerr << std::endl << "Syntax error: type --help for help" << std::endl << std::endl; exit(-1);
				}
			}			

			break;

			// set vocabulary and two html pages
			case 8:
            {
				std::string voc_parameter  = argv[2];
				std::string vocabularyFile = argv[3];
				std::string html1          = argv[4];
				std::string html1file      = argv[5];
				std::string html2          = argv[6];
				std::string html2file      = argv[7];

				if (voc_parameter == voc)
				{
					fileVocabularyName = vocabularyFile;		
				}
				else
				{
					std::cerr << std::endl << "Syntax error: type --help for help" << std::endl << std::endl; exit(-1);
				}

				if (html1 == htmlpage1)
				{
					htmlpagefilename1 = html1file;
				}
				else
				{
					std::cerr << std::endl << "Syntax error: type --help for help" << std::endl << std::endl; exit(-1);
				}

				if (html2 == htmlpage2)
				{
					htmlpagefilename2 = html2file;
				}
				else
				{
					std::cerr << std::endl << "Syntax error: type --help for help" << std::endl << std::endl; exit(-1);
				}
        	}

			break;

			default:
			{
				std::cerr << std::endl << "Syntax error: type --help for help" << std::endl << std::endl; exit(-1);
			}

			break;
		}

		std::cout << std::endl;
		std::cout << "Vocabulary  file name: " << fileVocabularyName << std::endl;
		std::cout << "Html Page 1 file name: " << htmlpagefilename1  << std::endl;
		std::cout << "Html Page 2 file name: " << htmlpagefilename2  << std::endl;

		// read vocabulary from file
	    Vocabulary vocabulary(fileVocabularyName, "", "r");

		// vocabulary word number
		unsigned int vocabulary_word_number = vocabulary.getVocabulary_vec().size();

		// print the vocabulary
		//std::cout << vocabulary << std::endl;

		// first and second html page
		HTMLPage htmlPage_1(htmlpagefilename1); // std::cout << htmlPage_1 << std::endl; std::cin.get();
		HTMLPage htmlPage_2(htmlpagefilename2); // std::cout << htmlPage_2 << std::endl; std::cin.get();

		// allocate bag of words of html page
		htmlPage_1.allocateBagOfWords(vocabulary_word_number);
		htmlPage_2.allocateBagOfWords(vocabulary_word_number);

		// processing the two html pages for create the bag of words of the two pages
		htmlPage_1.processing_for_create_bag_of_words(vocabulary);
		htmlPage_2.processing_for_create_bag_of_words(vocabulary);

		//std::cout << htmlPage_1 << std::endl; std::cin.get();
		//std::cout << htmlPage_2 << std::endl; std::cin.get();

		// get bag of words
		std::vector<double> bag_of_words_1 = htmlPage_1.getBag_of_words();
		std::vector<double> bag_of_words_2 = htmlPage_2.getBag_of_words();

		// compute cos theta, that is compute cos of angle between two vectors
		double cos_theta = compute_cos_theta(bag_of_words_1, bag_of_words_2);

		// print cos theta
		std::cout << std::endl << "COS THETA: " << cos_theta << std::endl << std::endl;

		// same indications
		std::cout << "If COS THETA = 1.0, then the two html pages are the same." << std::endl;
		std::cout << "If COS THETA = 0.0, then the two html pages are completely different." << std::endl << std::endl;

		// bound for the same contents
		std::cout << "BOUND FOR THE SAME CONTENT: 0.8 - 0.9" << std::endl << std::endl;
	}
	else
	{
		std::cerr << std::endl << "Syntax error: type --help for help" << std::endl << std::endl; exit(-1);
	}
}

double compute_cos_theta(const std::vector<double>& bag_of_words_1, const std::vector<double>& bag_of_words_2)
{
	// cos of angle between two vectors
	double cos_theta = 0.0;

	// number of vocabulary words
	unsigned int vocabulary_word_number = bag_of_words_1.size();
	assert (vocabulary_word_number == bag_of_words_2.size());

	// scalar product of two vectors
	double scalar_product = 0.0;

	// norma of first vector
	double norma_1 = 0.0;

	// norma of second vector
	double norma_2 = 0.0;

	unsigned int i;

	// compute scalar product
	for (i = 0; i < vocabulary_word_number; ++i)
	{
		scalar_product += (bag_of_words_1[i] * bag_of_words_2[i]);
	}

	// compute norma of first vector
	for (i = 0; i < vocabulary_word_number; ++i)
	{
		norma_1 += pow(bag_of_words_1[i], 2.0);
	}

	if (norma_1 == 0.0)
	{
		std::cerr << "All the words of the first html page aren't in the vocabulary." << std::endl; exit(-1);
	}

	norma_1 = sqrt(norma_1);

	// compute norma of second vector
	for (i = 0; i < vocabulary_word_number; ++i)
	{
		norma_2 += pow(bag_of_words_2[i], 2.0);
	}

	if (norma_2 == 0.0)
	{
		std::cerr << "All the words of the second html page aren't in the vocabulary." << std::endl; exit(-1);
	}

	norma_2 = sqrt(norma_2);

	// compute cos theta
	cos_theta = scalar_product / (norma_1 * norma_2);

	return cos_theta;
}
